`timescale 1ns / 1ps

`define NROUNDS_DIVIDED_BY_2 1
`define PCADD(a, b, mask) ((a)+(b))

module slow_key_setup1( clk, reset_n, n, data_out, ready_in, valid_in, ready_out, valid_out);

parameter INIT    = 3'b000;
parameter READN   = 3'b001;
parameter ROUNDS1 = 3'b010;
parameter ROUNDS2 = 3'b111;
parameter WRITEL  = 3'b011;
parameter WRITER  = 3'b100;
parameter WRITES1 = 3'b101;
parameter WRITES2 = 3'b110;

input clk, reset_n, valid_in, ready_out;
input [9:0] n;
output ready_in, valid_out;
output [7:0] data_out;

reg [7:0] data_out;
reg ready_in, valid_out;

reg [7:0] s [31:0];
reg [7:0] l;
reg [7:0] r;

reg [2:0] current_state, next_state;

reg [4:0] sindex;
reg [9:0] nreg;

always @ (posedge clk) begin 
   if (reset_n == 1'b0) 
		current_state <= INIT; 
   else 
		current_state <= next_state; 
end
 

always @ (current_state or valid_in or nreg or ready_out or sindex) 
   begin 
      case (current_state) 
      INIT: 
         if (valid_in) 
				next_state = READN;
			else
				next_state = INIT;
      READN:
			next_state = ROUNDS1;
		ROUNDS1:
			next_state = ROUNDS2;
		ROUNDS2:
			if(nreg == 0)
				next_state = WRITEL;
			else
				next_state = ROUNDS1;
		WRITEL:
			if(ready_out)
				next_state = WRITER;
			else
				next_state = WRITEL;
		WRITER:
			if(ready_out)
				next_state = WRITES1;
			else
				next_state = WRITER;
		WRITES1:
			if(ready_out)
				next_state = WRITES2;
			else
				next_state = WRITES1;
		WRITES2:
			if(sindex == 5'd0)
				next_state = INIT;
			else
				next_state = WRITES1;
      endcase 
   end

always @ (current_state or n or r or l or s or sindex or nreg) 
   begin
      case (current_state) 
      INIT: begin
			s[0] = 8'ha6;
			s[1] = 8'hac;
			s[2] = 8'hdb;
			s[3] = 8'hb7;
			s[4] = 8'hed;
			s[5] = 8'h96;
			s[6] = 8'h45;
			s[7] = 8'h99;
			s[8] = 8'h47;
			s[9] = 8'hf7;
			s[10] = 8'he2;
			s[11] = 8'h16;
			s[12] = 8'hd8;
			s[13] = 8'h69;
			s[14] = 8'ha3;
			s[15] = 8'h7e;
			s[16] = 8'he9;
			s[17] = 8'h44;
			s[18] = 8'h2e;
			s[19] = 8'h23;
			s[20] = 8'hb0;
			s[21] = 8'h7d;
			s[22] = 8'hb8;
			s[23] = 8'h66;
			s[24] = 8'h71;
			s[25] = 8'hff;
			s[26] = 8'h6c;
			s[27] = 8'he1;
			s[28] = 8'ha5;
			s[29] = 8'h29;
			s[30] = 8'h40;
			s[31] = 8'h3e;
			l = 8'd0;
			r = 8'd0;
			ready_in = 1;
			valid_out = 0;
			sindex = 0;
			nreg = 0;
		end
		READN: begin
			nreg = n;
			ready_in = 0;
		end
		ROUNDS1: begin: main_point
			integer i;
			for(i = 0; i < `NROUNDS_DIVIDED_BY_2; i=i+1) begin
				r = r ^ `PCADD(s[l[3:0]],s[16+(l>>4)], 8'h55);
				l = l ^ `PCADD(s[r[3:0]],s[16+(r>>4)], 8'haa);
			end
			s[sindex] = s[sindex] ^ l;
			s[sindex+1] = s[sindex+1] ^ r;
			if(sindex == 5'd30) begin
				nreg = nreg - 1;
				sindex = 0;
			end
			else
				sindex = sindex + 2;
		end: main_point
		ROUNDS2: begin
		end
		WRITEL: begin
			valid_out = 1;
			data_out = l;
		end
		WRITER: begin
			valid_out = 1;
			data_out = r;
			sindex = 0;
		end
		WRITES1: begin
			valid_out = 1;
			data_out = s[sindex];
		end
		WRITES2: begin
			valid_out = 0;
			sindex = sindex + 1;
		end
		endcase 
   end

endmodule
